package com.pj.current.netty.heart;

import cn.hutool.cache.CacheUtil;
import cn.hutool.cache.impl.TimedCache;
import cn.hutool.core.util.StrUtil;
import cn.hutool.extra.spring.SpringUtil;
import cn.hutool.json.JSONObject;
import cn.hutool.json.JSONUtil;
import com.pj.current.config.MyConfig;
import com.pj.current.netty.dto.ParamsDTO;
import com.pj.current.netty.dto.RequestPackageDTO;
import com.pj.current.netty.dto.ResponsePackageDTO;
import com.pj.project.tb_terminal.TbTerminalService;
import com.pj.utils.cache.RedisUtil;
import io.netty.channel.*;
import io.netty.handler.timeout.IdleState;
import io.netty.handler.timeout.IdleStateEvent;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;

import java.net.InetSocketAddress;
import java.util.concurrent.ConcurrentHashMap;

@Slf4j
@ChannelHandler.Sharable
@Component
public class HeartBeatHandler extends ChannelInboundHandlerAdapter {
    /**
     * 管理一个全局map，保存连接进服务端的通道数量
     */
    private static final ConcurrentHashMap<ChannelId, ChannelHandlerContext> CHANNEL_MAP = new ConcurrentHashMap<>();

    public static final TimedCache<String, String> STATE_CACHE = CacheUtil.newTimedCache(15000);


    /**
     * @param ctx
     * @DESCRIPTION: 有客户端连接服务器会触发此函数
     * @return: void
     */
    @Override
    public void channelActive(ChannelHandlerContext ctx) {
        //上线时同步时间===========


    }

    /**
     * @param ctx
     * @DESCRIPTION: 有客户端终止连接服务器会触发此函数
     * @return: void
     */
    @Override
    public void channelInactive(ChannelHandlerContext ctx) {

    }

    /**
     * @param ctx
     * @DESCRIPTION: 有客户端发消息会触发此函数
     * @return: void
     */
    @Override
    public void channelRead(ChannelHandlerContext ctx, Object msg) throws Exception {
        String body = JSONUtil.toJsonStr(msg);
        RequestPackageDTO dto = JSONUtil.toBean(body, RequestPackageDTO.class);
        log.info("心跳包:{}", body);
        MyConfig myConfig = SpringUtil.getBean(MyConfig.class);
        //心跳包
        String sn = JSONUtil.parseObj(dto.getPayload().getParams()).getStr("sn");
        STATE_CACHE.put(sn, sn);
        String commandStr = RedisUtil.get(myConfig.getHeartPrefix() + sn);
        if (StrUtil.isNotEmpty(commandStr)) {
            this.channelWrite(ctx, commandStr);
            RedisUtil.del(myConfig.getHeartPrefix() + sn);
            return;
        }
        //todo 检查是否存在库里
        TbTerminalService tbTerminalService = SpringUtil.getBean(TbTerminalService.class);
        InetSocketAddress insocket = (InetSocketAddress) ctx.channel().remoteAddress();
        String clientIp = insocket.getAddress().getHostAddress();
        tbTerminalService.initTerminal(sn, clientIp);
        ResponsePackageDTO responseDTO = new ResponsePackageDTO();
        ParamsDTO paramsDTO = new ParamsDTO();
        paramsDTO.setIp(myConfig.getIp()).setPort(myConfig.getCommandPort());
        ResponsePackageDTO.PayloadBean payloadBean = new ResponsePackageDTO.PayloadBean();
        payloadBean.setParams(paramsDTO);
        responseDTO.setPayload(payloadBean);
        String response = JSONUtil.toJsonStr(responseDTO);

        log.info("回复心跳:{}", response);
        this.channelWrite(ctx, response);
    }

    /**
     * @param msg 需要发送的消息内容
     * @DESCRIPTION: 服务端给客户端发送消息
     * @return: void
     */
    public void channelWrite(ChannelHandlerContext ctx, Object msg) throws Exception {
        //将客户端的信息直接返回写入ctx
        ctx.write(msg);
        //刷新缓存区
        ctx.flush();
    }

    @Override
    public void userEventTriggered(ChannelHandlerContext ctx, Object evt) throws Exception {

        String socketString = ctx.channel().remoteAddress().toString();

        if (evt instanceof IdleStateEvent) {
            IdleStateEvent event = (IdleStateEvent) evt;
            if (event.state() == IdleState.READER_IDLE) {
                log.info("Client: " + socketString + " READER_IDLE 读超时");
                ctx.disconnect();
            } else if (event.state() == IdleState.WRITER_IDLE) {
                log.info("Client: " + socketString + " WRITER_IDLE 写超时");
                ctx.disconnect();
            } else if (event.state() == IdleState.ALL_IDLE) {
                log.info("Client: " + socketString + " ALL_IDLE 总超时");
                ctx.disconnect();
            }
        }
    }

    /**
     * @param ctx
     * @DESCRIPTION: 发生异常会触发此函数
     * @return: void
     */
    @Override
    public void exceptionCaught(ChannelHandlerContext ctx, Throwable cause) throws Exception {

        System.out.println();
        ctx.close();
        log.info(ctx.channel().id() + " 发生了错误,此连接被关闭" + "此时连通数量: " + CHANNEL_MAP.size());
        //cause.printStackTrace();
    }

}
